@@ -679,14 +679,9 @@ OpFunctionEnd
 %v = OpVariable %_ptr_Function_v4float Function
 %17 = OpLoad %v4float %BaseColor
 OpStore %v %17
-OpSelectionMerge %18 None
-OpBranchConditional %true %19 %18
+OpBranch %19
 %19 = OpLabel
 OpKill
-%18 = OpLabel
-%23 = OpLoad %v4float %v
-OpStore %gl_FragColor %23
-OpReturn
 OpFunctionEnd
 )";
 
@@ -746,13 +741,9 @@ OpFunctionEnd
   const std::string after =
       R"(%foo_ = OpFunction %v4float None %9
 %19 = OpLabel
-OpSelectionMerge %20 None
-OpBranchConditional %true %21 %20
+OpBranch %21
 %21 = OpLabel
 OpReturnValue %13
-%20 = OpLabel
-%23 = OpUndef %v4float
-OpReturnValue %23
 OpFunctionEnd
 )";
 
@@ -1068,6 +1059,295 @@ OpFunctionEnd
       predefs + before, predefs + after, true, true);
 }
 
+TEST_F(DeadBranchElimTest, SwitchLiveCase) {
+  // #version 450
+  // 
+  // layout (location=0) in vec4 BaseColor;
+  // layout (location=0) out vec4 OutColor;
+  // 
+  // void main()
+  // {
+  //     switch (1) {
+  //       case 0:
+  //         OutColor = vec4(0.0,0.0,0.0,0.0);
+  //         break;
+  //       case 1:
+  //         OutColor = vec4(0.1,0.1,0.1,0.1);
+  //         break;
+  //       case 2:
+  //         OutColor = vec4(0.2,0.2,0.2,0.2);
+  //         break;
+  //       default:
+  //         OutColor = vec4(1.0,1.0,1.0,1.0);
+  //     }
+  // }
+
+  const std::string predefs =
+      R"(OpCapability Shader
+%1 = OpExtInstImport "GLSL.std.450"
+OpMemoryModel Logical GLSL450
+OpEntryPoint Fragment %main "main" %OutColor %BaseColor
+OpExecutionMode %main OriginUpperLeft
+OpSource GLSL 450
+OpName %main "main"
+OpName %OutColor "OutColor"
+OpName %BaseColor "BaseColor"
+OpDecorate %OutColor Location 0
+OpDecorate %BaseColor Location 0
+%void = OpTypeVoid
+%6 = OpTypeFunction %void
+%int = OpTypeInt 32 1
+%int_1 = OpConstant %int 1
+%float = OpTypeFloat 32
+%v4float = OpTypeVector %float 4
+%_ptr_Output_v4float = OpTypePointer Output %v4float
+%OutColor = OpVariable %_ptr_Output_v4float Output
+%float_0 = OpConstant %float 0
+%13 = OpConstantComposite %v4float %float_0 %float_0 %float_0 %float_0
+%float_0_1 = OpConstant %float 0.1
+%15 = OpConstantComposite %v4float %float_0_1 %float_0_1 %float_0_1 %float_0_1
+%float_0_2 = OpConstant %float 0.2
+%17 = OpConstantComposite %v4float %float_0_2 %float_0_2 %float_0_2 %float_0_2
+%float_1 = OpConstant %float 1
+%19 = OpConstantComposite %v4float %float_1 %float_1 %float_1 %float_1
+%_ptr_Input_v4float = OpTypePointer Input %v4float
+%BaseColor = OpVariable %_ptr_Input_v4float Input
+)";
+
+  const std::string before =
+      R"(%main = OpFunction %void None %6
+%21 = OpLabel
+OpSelectionMerge %22 None 
+OpSwitch %int_1 %23 0 %24 1 %25 2 %26
+%23 = OpLabel
+OpStore %OutColor %19
+OpBranch %22 
+%24 = OpLabel
+OpStore %OutColor %13
+OpBranch %22 
+%25 = OpLabel
+OpStore %OutColor %15
+OpBranch %22 
+%26 = OpLabel
+OpStore %OutColor %17
+OpBranch %22 
+%22 = OpLabel
+OpReturn
+OpFunctionEnd
+)";
+
+  const std::string after =
+      R"(%main = OpFunction %void None %6
+%21 = OpLabel
+OpBranch %25
+%25 = OpLabel
+OpStore %OutColor %15
+OpBranch %22
+%22 = OpLabel
+OpReturn
+OpFunctionEnd
+)";
+
+  SinglePassRunAndCheck<opt::DeadBranchElimPass>(
+      predefs + before, predefs + after, true, true);
+}
+
+TEST_F(DeadBranchElimTest, SwitchLiveDefault) {
+  // #version 450
+  // 
+  // layout (location=0) in vec4 BaseColor;
+  // layout (location=0) out vec4 OutColor;
+  // 
+  // void main()
+  // {
+  //     switch (7) {
+  //       case 0:
+  //         OutColor = vec4(0.0,0.0,0.0,0.0);
+  //         break;
+  //       case 1:
+  //         OutColor = vec4(0.1,0.1,0.1,0.1);
+  //         break;
+  //       case 2:
+  //         OutColor = vec4(0.2,0.2,0.2,0.2);
+  //         break;
+  //       default:
+  //         OutColor = vec4(1.0,1.0,1.0,1.0);
+  //     }
+  // }
+
+  const std::string predefs =
+      R"(OpCapability Shader
+%1 = OpExtInstImport "GLSL.std.450"
+OpMemoryModel Logical GLSL450
+OpEntryPoint Fragment %main "main" %OutColor %BaseColor
+OpExecutionMode %main OriginUpperLeft
+OpSource GLSL 450
+OpName %main "main"
+OpName %OutColor "OutColor"
+OpName %BaseColor "BaseColor"
+OpDecorate %OutColor Location 0
+OpDecorate %BaseColor Location 0
+%void = OpTypeVoid
+%6 = OpTypeFunction %void
+%int = OpTypeInt 32 1
+%int_7 = OpConstant %int 7
+%float = OpTypeFloat 32
+%v4float = OpTypeVector %float 4
+%_ptr_Output_v4float = OpTypePointer Output %v4float
+%OutColor = OpVariable %_ptr_Output_v4float Output
+%float_0 = OpConstant %float 0
+%13 = OpConstantComposite %v4float %float_0 %float_0 %float_0 %float_0
+%float_0_1 = OpConstant %float 0.1
+%15 = OpConstantComposite %v4float %float_0_1 %float_0_1 %float_0_1 %float_0_1
+%float_0_2 = OpConstant %float 0.2
+%17 = OpConstantComposite %v4float %float_0_2 %float_0_2 %float_0_2 %float_0_2
+%float_1 = OpConstant %float 1
+%19 = OpConstantComposite %v4float %float_1 %float_1 %float_1 %float_1
+%_ptr_Input_v4float = OpTypePointer Input %v4float
+%BaseColor = OpVariable %_ptr_Input_v4float Input
+)";
+
+  const std::string before =
+      R"(%main = OpFunction %void None %6
+%21 = OpLabel
+OpSelectionMerge %22 None 
+OpSwitch %int_7 %23 0 %24 1 %25 2 %26
+%23 = OpLabel
+OpStore %OutColor %19
+OpBranch %22 
+%24 = OpLabel
+OpStore %OutColor %13
+OpBranch %22 
+%25 = OpLabel
+OpStore %OutColor %15
+OpBranch %22 
+%26 = OpLabel
+OpStore %OutColor %17
+OpBranch %22 
+%22 = OpLabel
+OpReturn
+OpFunctionEnd
+)";
+
+  const std::string after =
+      R"(%main = OpFunction %void None %6
+%21 = OpLabel
+OpBranch %23
+%23 = OpLabel
+OpStore %OutColor %19
+OpBranch %22
+%22 = OpLabel
+OpReturn
+OpFunctionEnd
+)";
+
+  SinglePassRunAndCheck<opt::DeadBranchElimPass>(
+      predefs + before, predefs + after, true, true);
+}
+
+TEST_F(DeadBranchElimTest, SwitchLiveCaseBreakFromLoop) {
+  // This sample does not directly translate to GLSL/HLSL as
+  // direct breaks from a loop cannot be made from a switch.
+  // This construct is currently formed by inlining a function
+  // containing early returns from the cases of a switch. The
+  // function is wrapped in a one-trip loop and returns are
+  // translated to branches to the loop's merge block. 
+
+  const std::string predefs =
+      R"(OpCapability Shader
+%1 = OpExtInstImport "GLSL.std.450"
+OpMemoryModel Logical GLSL450
+OpEntryPoint Fragment %main "main" %OutColor %BaseColor
+OpExecutionMode %main OriginUpperLeft
+OpSource GLSL 450
+OpName %main "main"
+OpName %oc "oc"
+OpName %OutColor "OutColor"
+OpName %BaseColor "BaseColor"
+OpDecorate %OutColor Location 0
+OpDecorate %BaseColor Location 0
+%void = OpTypeVoid
+%7 = OpTypeFunction %void
+%bool = OpTypeBool
+%true = OpConstantTrue %bool
+%false = OpConstantFalse %bool
+%int = OpTypeInt 32 1
+%int_1 = OpConstant %int 1
+%float = OpTypeFloat 32
+%v4float = OpTypeVector %float 4
+%_ptr_Function_v4float = OpTypePointer Function %v4float
+%float_0 = OpConstant %float 0
+%17 = OpConstantComposite %v4float %float_0 %float_0 %float_0 %float_0
+%float_0_1 = OpConstant %float 0.1
+%19 = OpConstantComposite %v4float %float_0_1 %float_0_1 %float_0_1 %float_0_1
+%float_0_2 = OpConstant %float 0.2
+%21 = OpConstantComposite %v4float %float_0_2 %float_0_2 %float_0_2 %float_0_2
+%float_1 = OpConstant %float 1
+%23 = OpConstantComposite %v4float %float_1 %float_1 %float_1 %float_1
+%_ptr_Output_v4float = OpTypePointer Output %v4float
+%OutColor = OpVariable %_ptr_Output_v4float Output
+%_ptr_Input_v4float = OpTypePointer Input %v4float
+%BaseColor = OpVariable %_ptr_Input_v4float Input
+)";
+
+  const std::string before =
+      R"(%main = OpFunction %void None %7
+%26 = OpLabel
+%oc = OpVariable %_ptr_Function_v4float Function
+OpBranch %27
+%27 = OpLabel
+OpLoopMerge %28 %29 None
+OpBranch %30
+%30 = OpLabel
+OpSelectionMerge %31 None
+OpSwitch %int_1 %31 0 %32 1 %33 2 %34
+%32 = OpLabel
+OpStore %oc %17
+OpBranch %28
+%33 = OpLabel
+OpStore %oc %19
+OpBranch %28 
+%34 = OpLabel
+OpStore %oc %21
+OpBranch %28 
+%31 = OpLabel
+OpStore %oc %23
+OpBranch %28 
+%29 = OpLabel
+OpBranchConditional %false %27 %28
+%28 = OpLabel
+%35 = OpLoad %v4float %oc
+OpStore %OutColor %35
+OpReturn
+OpFunctionEnd
+)";
+
+  const std::string after =
+      R"(%main = OpFunction %void None %7
+%26 = OpLabel
+%oc = OpVariable %_ptr_Function_v4float Function
+OpBranch %27
+%27 = OpLabel
+OpLoopMerge %28 %29 None
+OpBranch %30
+%30 = OpLabel
+OpBranch %33
+%33 = OpLabel
+OpStore %oc %19
+OpBranch %28
+%29 = OpLabel
+OpBranchConditional %false %27 %28
+%28 = OpLabel
+%35 = OpLoad %v4float %oc
+OpStore %OutColor %35
+OpReturn
+OpFunctionEnd
+)";
+
+  SinglePassRunAndCheck<opt::DeadBranchElimPass>(
+      predefs + before, predefs + after, true, true);
+}
+
 // TODO(greg-lunarg): Add tests to verify handling of these cases:
 //
 //    More complex control flow
