@@ -19,18 +19,33 @@
 #include <cstring>
 #include <iostream>
 #include <sstream>
+#include <unordered_map>
+#include <unordered_set>
 #include <vector>
 
 #include "spirv/1.2/spirv.h"
 #include "source/enum_string_mapping.h"
+#include "source/comp/markv_model.h"
 #include "source/opcode.h"
 #include "source/operand.h"
 #include "source/spirv_constant.h"
+#include "source/util/huffman_codec.h"
 
 using libspirv::SpirvStats;
+using spvutils::HuffmanCodec;
 
 namespace {
 
+// Signals that the value is not in the coding scheme and a fallback method
+// needs to be used.
+const uint64_t kMarkvNoneOfTheAbove =
+    spvtools::MarkvModel::GetMarkvNoneOfTheAbove();
+
+inline uint32_t CombineOpcodeAndNumOperands(uint32_t opcode,
+                                            uint32_t num_operands) {
+  return opcode | (num_operands << 16);
+}
+
 // Returns all SPIR-V v1.2 opcodes.
 std::vector<uint32_t> GetAllOpcodes() {
   return std::vector<uint32_t>({
@@ -599,23 +614,30 @@ void StatsAnalyzer::WriteCodegenOpcodeAndNumOperandsHist(std::ostream& out) {
     total += kv.second;
   }
 
+  uint32_t left_out = 0;
+
   for (const auto& kv : stats_.opcode_and_num_operands_hist) {
     const uint32_t count = kv.second;
     const double kFrequentEnoughToAnalyze = 0.001;
-    if (double(count) / double(total) < kFrequentEnoughToAnalyze) continue;
     const uint32_t opcode_and_num_operands = kv.first;
     const uint32_t opcode = opcode_and_num_operands & 0xFFFF;
     const uint32_t num_operands = opcode_and_num_operands >> 16;
 
-    if (opcode == SpvOpTypeStruct)
+    if (opcode == SpvOpTypeStruct ||
+        double(count) / double(total) < kFrequentEnoughToAnalyze) {
+      left_out += count;
       continue;
+    }
 
     out << "    { CombineOpcodeAndNumOperands(SpvOp"
         << spvOpcodeString(SpvOp(opcode))
         << ", " << num_operands << "), " << count << " },\n";
   }
 
-  out << "    { kMarkvNoneOfTheAbove, " << 1 + int(total * 0.05) << " },\n";
+  // Heuristic.
+  const uint32_t none_of_the_above =
+      std::max(1, int(left_out + total * 0.01));
+  out << "    { kMarkvNoneOfTheAbove, " << none_of_the_above << " },\n";
   out << "  });\n}\n";
 }
 
@@ -638,10 +660,9 @@ void StatsAnalyzer::WriteCodegenOpcodeAndNumOperandsMarkovHuffmanCodecs(
       total += pair.second;
     }
 
-    out << "  {\n";
-    out << "    std::unique_ptr<HuffmanCodec<uint64_t>> "
-        << "codec(new HuffmanCodec<uint64_t>({\n";
+    uint32_t left_out = 0;
 
+    std::map<uint64_t, uint32_t> processed_hist;
     for (const auto& pair : hist) {
       const uint32_t opcode_and_num_operands = pair.first;
       const uint32_t opcode = opcode_and_num_operands & 0xFFFF;
@@ -654,17 +675,25 @@ void StatsAnalyzer::WriteCodegenOpcodeAndNumOperandsMarkovHuffmanCodecs(
       const double posterior_freq = double(count) / double(total);
 
       if (opcode_freq_[opcode] < kFrequentEnoughToAnalyze &&
-          posterior_freq < kFrequentEnoughToAnalyze) continue;
-
-      total += count;
-      out << "      { CombineOpcodeAndNumOperands(SpvOp"
-          << spvOpcodeString(SpvOp(opcode))
-          << ", " << num_operands << "), " << count << " },\n";
+          posterior_freq < kFrequentEnoughToAnalyze) {
+        left_out += count;
+        continue;
+      }
+      processed_hist.emplace(CombineOpcodeAndNumOperands(opcode, num_operands),
+                             count);
     }
 
-    out << "      { kMarkvNoneOfTheAbove, " << 1 + int(total * 0.05) << " },\n";
+    // Heuristic.
+    processed_hist.emplace(kMarkvNoneOfTheAbove,
+                           std::max(1, int(left_out + total * 0.01)));
 
-    out << "    }));\n" << std::endl;
+    HuffmanCodec<uint64_t> codec(processed_hist);
+
+    out << "  {\n";
+    out << "    std::unique_ptr<HuffmanCodec<uint64_t>> "
+        << "codec(new HuffmanCodec<uint64_t>";
+    out << codec.SerializeToText(4);
+    out << ");\n" << std::endl;
     out << "    codecs.emplace(SpvOp" << GetOpcodeString(prev_opcode)
         << ", std::move(codec));\n";
     out << "  }\n\n";
@@ -695,22 +724,31 @@ void StatsAnalyzer::WriteCodegenLiteralStringHuffmanCodecs(std::ostream& out) {
       total += pair.second;
     }
 
-    out << "  {\n";
-    out << "    std::unique_ptr<HuffmanCodec<std::string>> "
-        << "codec(new HuffmanCodec<std::string>({\n";
+    uint32_t left_out = 0;
+
+    std::map<std::string, uint32_t> processed_hist;
     for (const auto& pair : hist) {
       const uint32_t count = pair.second;
       const double freq = double(count) / double(total);
       const double kStringFrequentEnoughToAnalyze = 0.001;
-      if (freq < kStringFrequentEnoughToAnalyze) continue;
-      out << "      { std::string(\"" << pair.first << "\"), " << count
-          << " },\n";
+      if (freq < kStringFrequentEnoughToAnalyze) {
+        left_out += count;
+        continue;
+      }
+      processed_hist.emplace(pair.first, count);
     }
 
-    out << "      { std::string(\"kMarkvNoneOfTheAbove\"), "
-        << 1 + int(total * 0.05) << " },\n";
+    // Heuristic.
+    processed_hist.emplace("kMarkvNoneOfTheAbove",
+                           std::max(1, int(left_out + total * 0.01)));
 
-    out << "    }));\n" << std::endl;
+    HuffmanCodec<std::string> codec(processed_hist);
+
+    out << "  {\n";
+    out << "    std::unique_ptr<HuffmanCodec<std::string>> "
+        << "codec(new HuffmanCodec<std::string>";
+    out << codec.SerializeToText(4);
+    out << ");\n" << std::endl;
     out << "    codecs.emplace(SpvOp" << spvOpcodeString(SpvOp(opcode))
         << ", std::move(codec));\n";
     out << "  }\n\n";
@@ -741,21 +779,32 @@ void StatsAnalyzer::WriteCodegenNonIdWordHuffmanCodecs(std::ostream& out) {
       total += pair.second;
     }
 
-    out << "  {\n";
-    out << "    std::unique_ptr<HuffmanCodec<uint64_t>> "
-        << "codec(new HuffmanCodec<uint64_t>({\n";
+    uint32_t left_out = 0;
+
+    std::map<uint64_t, uint32_t> processed_hist;
     for (const auto& pair : hist) {
       const uint32_t word = pair.first;
       const uint32_t count = pair.second;
       const double freq = double(count) / double(total);
-      const double kWordFrequentEnoughToAnalyze = 0.001;
-      if (freq < kWordFrequentEnoughToAnalyze) continue;
-      out << "      { " << word << ", " << count << " },\n";
+      const double kWordFrequentEnoughToAnalyze = 0.003;
+      if (freq < kWordFrequentEnoughToAnalyze) {
+        left_out += count;
+        continue;
+      }
+      processed_hist.emplace(word, count);
     }
 
-    out << "      { kMarkvNoneOfTheAbove, " << 1 + int(total * 0.05) << " },\n";
+    // Heuristic.
+    processed_hist.emplace(kMarkvNoneOfTheAbove,
+                           std::max(1, int(left_out + total * 0.01)));
 
-    out << "    }));\n" << std::endl;
+    HuffmanCodec<uint64_t> codec(processed_hist);
+
+    out << "  {\n";
+    out << "    std::unique_ptr<HuffmanCodec<uint64_t>> "
+        << "codec(new HuffmanCodec<uint64_t>";
+    out << codec.SerializeToText(4);
+    out << ");\n" << std::endl;
     out << "    codecs.emplace(std::pair<uint32_t, uint32_t>(SpvOp"
         << spvOpcodeString(SpvOp(opcode))
         << ", " << index << "), std::move(codec));\n";
@@ -773,36 +822,52 @@ void StatsAnalyzer::WriteCodegenIdDescriptorHuffmanCodecs(
       << "  std::map<std::pair<uint32_t, uint32_t>, "
       << "std::unique_ptr<HuffmanCodec<uint64_t>>> codecs;\n";
 
+  std::unordered_set<uint32_t> descriptors_with_coding_scheme;
+
   for (const auto& kv : stats_.operand_slot_id_descriptor_hist) {
     const auto& opcode_and_index = kv.first;
     const uint32_t opcode = opcode_and_index.first;
     const uint32_t index = opcode_and_index.second;
 
-    const double kOpcodeFrequentEnoughToAnalyze = 0.001;
+    const double kOpcodeFrequentEnoughToAnalyze = 0.003;
     if (opcode_freq_[opcode] < kOpcodeFrequentEnoughToAnalyze) continue;
 
     const std::map<uint32_t, uint32_t>& hist = kv.second;
 
+
     uint32_t total = 0;
     for (const auto& pair : hist) {
       total += pair.second;
     }
 
-    out << "  {\n";
-    out << "    std::unique_ptr<HuffmanCodec<uint64_t>> "
-        << "codec(new HuffmanCodec<uint64_t>({\n";
+    uint32_t left_out = 0;
+
+    std::map<uint64_t, uint32_t> processed_hist;
     for (const auto& pair : hist) {
       const uint32_t descriptor = pair.first;
       const uint32_t count = pair.second;
       const double freq = double(count) / double(total);
-      const double kDescriptorFrequentEnoughToAnalyze = 0.005;
-      if (freq < kDescriptorFrequentEnoughToAnalyze) continue;
-      out << "      { " << descriptor << ", " << count << " },\n";
+      const double kDescriptorFrequentEnoughToAnalyze = 0.003;
+      if (freq < kDescriptorFrequentEnoughToAnalyze) {
+        left_out += count;
+        continue;
+      }
+      processed_hist.emplace(descriptor, count);
+      descriptors_with_coding_scheme.insert(descriptor);
     }
 
-    out << "      { kMarkvNoneOfTheAbove, " << 1 + int(total * 0.05) << " },\n";
+    // Heuristic.
+    processed_hist.emplace(kMarkvNoneOfTheAbove,
+                           std::max(1, int(left_out + total * 0.01)));
 
-    out << "    }));\n" << std::endl;
+
+    HuffmanCodec<uint64_t> codec(processed_hist);
+
+    out << "  {\n";
+    out << "    std::unique_ptr<HuffmanCodec<uint64_t>> "
+        << "codec(new HuffmanCodec<uint64_t>";
+    out << codec.SerializeToText(4);
+    out << ");\n" << std::endl;
     out << "    codecs.emplace(std::pair<uint32_t, uint32_t>(SpvOp"
         << spvOpcodeString(SpvOp(opcode))
         << ", " << index << "), std::move(codec));\n";
@@ -810,4 +875,12 @@ void StatsAnalyzer::WriteCodegenIdDescriptorHuffmanCodecs(
   }
 
   out << "  return codecs;\n}\n";
+
+  out << "\nstd::unordered_set<uint32_t> GetDescriptorsWithCodingScheme() {\n"
+      << "  std::unordered_set<uint32_t> descriptors_with_coding_scheme = {\n";
+  for (uint32_t descriptor : descriptors_with_coding_scheme) {
+    out << "    " << descriptor << ",\n";
+  }
+  out << "  };\n";
+  out << "  return descriptors_with_coding_scheme;\n}\n";
 }
